from __future__ import absolute_import

from imagekitio.image import Image
from imagekitio.upload import Upload
from sys import exit
from imagekitio.utils import (PY3, to_bytes, to_bytearray, to_string, string_types, unquote, urlencode, Error)


class Imagekit(object):
    def __init__(self, args=None):
        if args:
            if PY3:
                # self.globl = dict(get_defaults().items() | options.items())
                self.globl = dict(get_defaults(), **args)
            else:
                self.globl = dict(get_defaults().items() + args.items())
        else:
            self.globl = get_defaults()

        if not verify(self.globl):
            # print("ImageKit Id, API Key and API secret are necessary for initialization.")
            raise Error("ImageKit Id, API Key and API secret are necessary for initialization. - %d - %s" % (2400, "BAD_REQUEST"))
            exit()

    def __str__(self):
        return str(self.__class__) + ": " + str(self.__dict__)

    ''' For generating image URLs for fetching the image '''
    def image(self, key, options, signed_url=None):
        img = Image(key, options, self.globl)
        if signed_url:
            resp = img.signed_url(signed_url)
        else:
            resp = img.url()

        return resp

    ''' For uploading the images to your imagekit account '''
    def upload(self, image, options):
        print(options)
        try:
            if not options['filename'] or options['filename'] == "":
                raise Error("Invalid or missing upload parameters - %d - %s" % (2400, "BAD_REQUEST"))
                exit()
        except:
            raise Error("Invalid or missing upload parameters - %d - %s" % (2400, "BAD_REQUEST"))
            exit()

        up = Upload(image, options, self.globl)
        resp = up.uploader()

        return resp.json()


def get_defaults():
    return {
        "imagekit_id": "",
        "api_key": "",
        "api_secret": "",
        "use_subdomain": None,
        "use_secure": None
    }


def verify(globl):
    if globl['imagekit_id'] and globl['api_key'] and globl['api_secret']:
        return True
    else:
        return False
